@@ -63,7 +63,7 @@ my $DIR_TYPE = 1;
 my $FILE_TYPE = 2;
 
 use vars qw($VERSION);
-$VERSION = "0.907";
+$VERSION = "0.908";
 #
 # Descriptions partially based on mapitags.h
 #
@@ -109,6 +109,7 @@ my $skipproperties = {
   # Content properties
   '1008' => "Summary or something",
   '1009' => "RTF Compressed",
+  '10F3' => "URL component name",
   # --
   '1046' => "From address variant",
   # 'Common property'
@@ -124,7 +125,8 @@ my $skipproperties = {
   '370A' => "Tag identifying application that supplied the attachment",
   '3713' => "Icon URL?",
   # 'Mail user'
-  '3A20' => "Address variant",
+  '3A00' => "Recipient's account name",
+  '3A20' => "Recipient's display name",
   # 3900 -- 39FF: 'Address book'
   '39FF' => "7 bit display name",
   # 'Display table properties'
@@ -189,6 +191,7 @@ my $MAP_SUBITEM_FILE = {
   '0E04' => "TO",              # To: Names
   '0E03' => "CC",              # Cc: Names
   '1035' => "MESSAGEID",       # Message-Id
+  '1039' => "REFERENCES",      # References: Header
   '1042' => "INREPLYTO",       # In reply to Message-Id
 };
 
@@ -201,6 +204,13 @@ my $MAP_ADDRESSITEM_FILE = {
   '39FE' => "SMTPADDRESS",  # SMTP Address variant
 };
 
+my $MAP_PROPSTREAM_TAG = {
+    0x3007 => 'DATE2ND',    # Outlook created??
+    0x0039 => 'DATE1ST',    # Outlook sent date
+ #  0x0E06 => 'DATE2ND',    # more dates, not needed here
+ #  0x3008 => 'DATE2ND',
+};
+
 #
 # Main body of module
 #
@@ -318,7 +328,7 @@ sub _process_root_dir {
     if ($child->{Type} == $DIR_TYPE) {
       $self->_process_subdirectory($child);
     } elsif ($child->{Type} == $FILE_TYPE) {
-      $self->_process_pps_file_entry($child, $self, $MAP_SUBITEM_FILE);
+      $self->_process_pps_file_entry($child, $self, $MAP_SUBITEM_FILE, $MAP_PROPSTREAM_TAG);
     } else {
       carp "Unknown entry type: $child->{Type}";
     }
@@ -451,13 +461,36 @@ sub _process_attachment_subdirectory {
   return;
 }
 
+sub _process_prop_stream {
+  my ($self, $target, $data, $map) = @_;
+  my ($n, $len) = (32, length $data) ;
+
+  while ($n + 16 <= $len) {
+    my @f = unpack "v4", substr $data, $n, 8;
+    my $t = $map->{$f[1]} ;
+    # $f[2]: bit 1 -- mandatory, bit 2 -- readable, bit 3 -- writable
+    next unless $t and ($f[2] & 2) and $f[3] == 0;
+
+    # At the moment, there are only date entries ...
+    my @a = OLE::Storage_Lite::OLEDate2Local substr $data, $n + 8, 8;
+
+    if ($t eq 'DATE1ST') {
+      unshift @{$target->{PROPDATE}}, $self->_format_date(\@a) ;
+    } else { # DATE2ND
+      push @{$target->{PROPDATE}}, $self->_format_date(\@a) ;
+    }
+  } continue {
+    $n += 16 ;
+  }
+}
+
 #
 # Generic processor for a file entry: Inserts the entry's data into the
 # hash $target, using the $map to find the proper key.
 # TODO: Mapping should probably be applied at a later time instead.
 #
 sub _process_pps_file_entry {
-  my ($self, $pps, $target, $map) = @_;
+  my ($self, $pps, $target, $map, $map2) = @_;
 
   my $name = $self->_get_pps_name($pps);
   my ($property, $encoding) = $self->_parse_item_name($name);
@@ -475,6 +508,8 @@ sub _process_pps_file_entry {
       $data =~ s/\r\n/\n/sg;
     }
     $target->{$key} = $data;
+  } elsif ($name eq '__properties_version1 0' and $map2) {
+    $self->_process_prop_stream ($target, $pps->{Data}, $map2);
   } else {
     $self->_warn_about_unknown_file($pps);
   }
@@ -560,8 +595,8 @@ sub _extract_ole_date {
     # Make Date
     my $datearr;
     $datearr = $pps->{Time2nd};
-    $datearr = $pps->{Time1st} unless($datearr);
-    $self->{OLEDATE} = $self->_format_date($datearr) if $datearr;
+    $datearr = $pps->{Time1st} unless $datearr and @$datearr[0];
+    $self->{OLEDATE} = $self->_format_date($datearr) if $datearr and @$datearr[0];
   }
   return;
 }
@@ -747,6 +782,7 @@ sub _SetHeaderFields {
   $self->_AddHeaderField($mime, 'Cc', $self->_ExpandAddressList($self->{CC}));
   $self->_AddHeaderField($mime, 'Message-Id', $self->{MESSAGEID});
   $self->_AddHeaderField($mime, 'In-Reply-To', $self->{INREPLYTO});
+  $self->_AddHeaderField($mime, 'References', $self->{REFERENCES});
 
   # Least preferred option to set the Date: header; this uses the date the
   # msg file was saved.
@@ -755,6 +791,9 @@ sub _SetHeaderFields {
   # Second preferred option: get it from the SUBMISSION_ID:
   $self->_AddHeaderField($mime, 'Date', $self->_submission_id_date());
 
+  # Most prefered option from the property list
+  $self->_AddHeaderField($mime, 'Date', $self->{PROPDATE}->[0]);
+
   # After this, we'll try getting the date from the original headers.
   return;
 }
